/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023

 Date          : February 2009

 File          : SPI_Slave.c

 Hardware      : Applicable to ADuC7023 rev A later silicon
                 
 Description   : This SPI Slave example shows how to initialise the SPI peripheral in Slave mode.
 			This simple example expects to receive 6 bytes.
			After this, it will return these 6 bytes when it receives clocks from the Master
				

*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT22 0x400000
#define BIT23 0x800000
#define BIT25 0x2000000
#define BIT30 0x40000000
#define BIT31 0x80000000

#include <aduc7023.h>

unsigned char szStoreData[] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};	// Array to send to Slave
unsigned char ucStoreCount = 0;  // Array index variable for szStoreData[]
unsigned char ucWaitingForXIRQ0 = 1; // Flag to begin write from Slave - controlled by XIRQ0
unsigned char ucReadOnly = 0;
unsigned char ucTxCnt = 0;
unsigned char ucbuffer = 0;
int main(void) 
{
	POWKEY1 = 0x01;								// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	

	POWKEY3 = 0x76;			 					// Enable PWM, SPI, I2C0, I2C1 Peripherals with 41.78MHz clock
    POWCON1 = 0x924;					
	POWKEY4 = 0xB1;

	// Configure P0.6 as MISO, P0.7 as MOSI, 
	// Configure P1.0 as SCLOCK and P1.1 as /SS
	GP0CON |= 0x11000000;						// Only write to bits 7,6
	GP1CON |= 0x11;								// Only write to bits 1,0
	// Enable SPI Slave mode and interrupt sources
	SPICON = BIT0    							// Enable SPI 
		   + BIT7   							// Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
		   + BIT9								// Set Bit 9 to operate as normal
		   + BIT11;		   						//Continuous transfer mode + Enable IRQ on 1 byte transfer
	IRQEN = BIT12; 								// Enable SPI + XIRQ0 interrupt
	// Wait for Master Transmit sequence
	ucStoreCount = 0;
	while (1)
	{

	}			
}
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int SPIMSTATUS = 0;

	IRQSTATUS = IRQSTA;	   						// Read off IRQSTA register

	if ((IRQSTATUS & BIT12) == BIT12)			//If SPI Slave interrupt source
	{
	   SPIMSTATUS = SPISTA;
	   if ((SPIMSTATUS & BIT4) == BIT4) 		// If SPI Slave Tx Underflow
	   {
			SPITX = 0x55;
	   }

	   if ((SPIMSTATUS & BIT5) == BIT5) 		// If SPI Slave Tx IRQ
	   {
			if (ucStoreCount < 6)		   		// Have max 6 bytes been sent?
				SPITX = 0x55;
			if (ucStoreCount >=6)
				ucStoreCount = 0;
	   }
	   if ((SPIMSTATUS & BIT6) == BIT6) 		// If SPI Slave Rx IRQ
	   {
			if (ucStoreCount < 6)		   		// Have max 6 bytes been received?
			{
				if ( ucReadOnly == 0)
				{
					szStoreData[ucStoreCount] = SPIRX;  // Read Rx buffer
					SPITX = szStoreData[ucStoreCount];
					ucbuffer = szStoreData[ucStoreCount];
					ucStoreCount++;
					if (ucStoreCount >=6)
					{
						ucStoreCount = 0;
						ucReadOnly = 1;
						SPICON |= BIT13;		// Flush Tx buffer
						SPICON &= ~BIT13;
					    ucbuffer = szStoreData[ucStoreCount++];
						SPITX = ucbuffer;
					}
				}
				else  // Read from slave only
				{
					ucbuffer = szStoreData[ucStoreCount++];
					SPITX = ucbuffer;
					ucTxCnt++;
				}
			}
	   }
	   // Test for SPICON.8/SPISTA.7 - remove reading fo SPIRX register
	   if ((SPIMSTATUS & BIT7) == BIT7) 		// If SPI Slave Rx FIFO Overflow status IRQ
	   {

	   }
	}
}
